# -*- coding: utf-8 -*-

WIKI_HOST = ''
PROJECT_HOST = 'prom.ua'
PROJECT_PORT = ''
GRID_HOST_PORT = '0.0.0.0:13400'
DB = ''
CHECK_TEST_CASE = False

OPERA_BINARY_LOCATION = '/usr/bin/opera'

# for windows 7
# OPERA_BINARY_LOCATION = r'C:\Program Files\Opera\launcher.exe'

# for windows 10
# import os
# OPERA_BINARY_LOCATION = (
#     r'C:\Users\{}\AppData\Local\Programs\Opera\launcher.exe'.format(
#         os.environ.get('USERNAME')
#     )
# )
